#!/usr/bin/env python
#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
This script converts .ldv bundles by moving the following to the property db:
  - title in __content__
  - mtime (last modified time)

Previous __content__ files are backed up as content.bak
"""

import os
import sys
import re
import shutil

# adjust accordingly if need be
sys.path.append("../lib/")

import filesystem

def usage():
    print 'Usage:', sys.argv[0], '<lfs_root>'
    sys.exit(0)


if len(sys.argv) != 2:
    usage()

lfs_root = sys.argv[1]

lfs = filesystem.LeonardoFileSystem(lfs_root)

def convert(lfs, key=""):
    directories, files = lfs.get_children(key)
    for filename in files:
        f = lfs.get(key + "/" + filename)
        print f.key
        content = f.get_content()
        content_type = f.get_content_type()
        files = f.get_content_files_()
        ST_MTIME = 8
        if files:
            last_mod = os.stat(files[0])[ST_MTIME]
        else:
            last_mod = None
        if content_type in ["wiki04", "wikiBNL"]:
            m = re.match(r'==([^=]+)==', content)
            if m:
                title = m.group(1)
                rest = content[m.end(1)+2:]
                print "TITLE:", title
                shutil.copy2(f.get_content_filename_(content_type), f.get_directory_() + "content.bak")
                f.set_content(rest, content_type)
                print "SET NEW CONTENT"
                f.set_property("page_title", title)
                print "SET PAGE TITLE"
            else:
                print "NO TITLE"
        else:
            print "NOT WIKI", content_type
        if last_mod:
            f.set_property("last_modified", last_mod)
            print "SET LAST MODIFIED PROPERTY"
        
    for directory in directories:
        convert(lfs, key + "/" + directory)


convert(lfs)