#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


__provider__      = "AtomProvider"
__contributions__ = {"head_links": "head_link"}    


import time
import urllib

from web_resource import TopLevelResource

from providers.blog.blog_provider import Walker, ALL

import templates


def head_link(request, resource, main_resource, config):

    # @@@ this should probably only be added on certain pages

    atom_href = config.get("DEFAULT", "home_href") + config.get("atom", "full_atom_path")
    blog_title = config.get("blog", "blog_title")

    return templates.auto_discovery_link % locals()


class AtomProvider:
    

    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config
       
        self.atom_path          = config.get("atom", "atom_path")
        self.full_atom_path     = config.get("atom", "full_atom_path")
        self.atom_all_path      = config.get("atom", "atom_all_path")
        self.full_atom_all_path = config.get("atom", "full_atom_all_path")
        self.atom_recent_count  = config.getint("atom", "atom_recent_count")

       
    def get(self, resource_id, request, main_resource):

        if resource_id == self.atom_path:
            return AtomResource(resource_id, self.resource_manager, self.lfs, request, self.config, full=False, number=self.atom_recent_count)
        elif resource_id == self.full_atom_path:
            return AtomResource(resource_id, self.resource_manager, self.lfs, request, self.config, full=True, number=self.atom_recent_count)
        elif resource_id == self.atom_all_path:
            return AtomResource(resource_id, self.resource_manager, self.lfs, request, self.config, full=False, number=ALL)
        elif resource_id == self.full_atom_all_path:
            return AtomResource(resource_id, self.resource_manager, self.lfs, request, self.config, full=True, number=ALL)
        else:
            return None


    def post(self, resource_id, request, main_resource):
        return None
        


class AtomResource(TopLevelResource):


    def __init__(self, resource_id, resource_manager, lfs, request, config, full, number):
        TopLevelResource.__init__(self, resource_id, "text/xml; charset=utf-8")
        
        if full:
            formatter = FullAtomFormatter(resource_manager, config)
        else:
            formatter = AtomFormatter(resource_manager, config)
        walker = Walker(formatter, config.get("blog", "blog_path"), lfs)
        if number == ALL:
            self.content = walker.all(request, self)
        else:
            self.content = walker.recent(request, self, number)
        
    def get_content(self):
        return self.content
    
    

class AtomFormatter:


    def __init__(self, resource_manager, config):
    
        self.resource_manager = resource_manager
        
        self.atom_path   = config.get("atom", "atom_path")
        self.site_url    = config.get("blog", "site_url")
        self.blog_title  = config.get("blog", "blog_title")
        self.blog_path   = config.get("blog", "blog_path")
        self.blog_author = config.get("atom", "blog_author")
        
        self.shown_modified = False
        
        
    def header(self):
        blog_title  = self.blog_title
        blog_url    = self.site_url + self.blog_path
        blog_author = self.blog_author
        
        return templates.atom_header % locals()


    def footer(self):
        return templates.atom_footer % locals()
        

    def no_entries(self):
        return templates.atom_no_entries % locals()
        
        
    def entry(self, year, month, day, entry, request, main_resource):
        page_key = "%s/%s/%s/%s/%s" % (self.blog_path, year, month, day, entry)
        page = self.resource_manager.get(page_key, request, main_resource)
        iso_time = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime(page.get_lastmod()))
        title = page.get_title()
        html_link = self.site_url + urllib.quote(page_key)
        
        if not self.shown_modified:
            self.shown_modified = True
            s = templates.atom_modified % locals()
            # this is the overall feed lastmod, so tell the main resource
            main_resource.lastmod = time.gmtime(page.get_lastmod())
        else:
            s = ""
    
        modified = iso_time
        issued = iso_time
            
        s += templates.atom_entry % locals()
        return s   



class FullAtomFormatter(AtomFormatter):

        
    def entry(self, year, month, day, entry, request, main_resource):
        page_key = "%s/%s/%s/%s/%s" % (self.blog_path, year, month, day, entry)
        page = self.resource_manager.get(page_key, request, main_resource)
        iso_time = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime(page.get_lastmod()))
        title = page.get_title()
        html_link = self.site_url + urllib.quote(page_key)
                
        if not self.shown_modified:
            self.shown_modified = True
            s = templates.atom_modified % locals()
            # this is the overall feed lastmod, so tell the main resource
            main_resource.lastmod = time.gmtime(page.get_lastmod())
        else:
            s = ""
    
        modified = iso_time
        issued = iso_time
            
        content = page.get_html()
        s += templates.atom_full_entry % locals()
        return s   
        