#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


__provider__      = "BlogProvider"
__contributions__ = {}    


from web_resource import PagePart

from providers.page.page_provider import HTMLPage, HTMLPage404

import templates


ALL = "all"
YEARS = ["2004", "2005"]
MONTHS = ["01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"]
DAYS = [
    "01", "02", "03", "04", "05", "06", "07", "08", "09", "10",
    "11", "12", "13", "14", "15", "16", "17", "18", "19", "20",
    "21", "22", "23", "24", "25", "26", "27", "28", "29", "30",
    "31"]



class BlogProvider:
    
    
    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config
        
        self.blog_path         = config.get("blog", "blog_path")
        self.all_blog_path     = config.get("blog", "all_blog_path")
        self.site_url          = config.get("blog", "site_url")
        self.blog_recent_count = config.getint("blog", "blog_recent_count")
        
       
    def get(self, resource_id, request, main_resource):
 
        if resource_id == self.all_blog_path:
            page_part = BlogPagePart(resource_id, self.resource_manager, request, self.lfs, self.config, number=ALL)
            return HTMLPage(resource_id, page_part, self.resource_manager, request, main_resource, self.config)
    
        if not resource_id.startswith(self.blog_path):
            return None

        blog_path = resource_id[len(self.blog_path):]
        
        year, month, day, entry = None, None, None, None
        path_components = blog_path.split("/")
        
        if len(path_components) == 1:
            pass
        elif len(path_components) == 2:
            year = path_components[1]
        elif len(path_components) == 3:
            year, month = path_components[1:]
        elif len(path_components) == 4:
            year, month, day = path_components[1:]
        elif len(path_components) == 5:
            year, month, day, entry = path_components[1:]
        else:
            return HTMLPage404(resource_id, self.resource_manager, request, main_resource, self.config)

        if year and year not in YEARS:
            return HTMLPage404(resource_id, self.resource_manager, request, main_resource, self.config)
        if month and month not in MONTHS:
            return HTMLPage404(resource_id, self.resource_manager, request, main_resource, self.config)
        if day and day not in DAYS:
            return HTMLPage404(resource_id, self.resource_manager, request, main_resource, self.config)

        if entry:
            return None # we don't need to handle it @@@ although we could to add date/permalink
        else:
            if year:
                page_part = BlogPagePart(resource_id, self.resource_manager, request, self.lfs, self.config, year, month, day)
            else:
                if main_resource != None:  # if not a top-level request,
                    return None            # don't handle it here
                page_part = TopBlogPagePart(resource_id, self.resource_manager, request, self.lfs, self.config, number=self.blog_recent_count)
            return HTMLPage(resource_id, page_part, self.resource_manager, request, main_resource, self.config)


    def post(self, resource_id, resource, main_resource):
        return None
        


class BlogPagePart(PagePart):


    def __init__(self, resource_id, resource_manager, request, lfs, config, year=None, month=None, day=None, number=None):
        PagePart.__init__(self, resource_id)

        formatter = HTMLFormatter(resource_manager, config)
        walker = Walker(formatter, config.get("blog", "blog_path"), lfs)

        blog_title = config.get("blog", "blog_title")
        
        if day:
            title = templates.blog_title_day % locals()
            content = walker.day(year, month, day, request, self)
        elif month:
            title = templates.blog_title_month % locals()
            content = walker.month(year, month, request, self)
        elif year:
            title = templates.blog_title_year % locals()
            content = walker.year(year, request, self)
        elif number == ALL:
            title = templates.blog_title_all % locals()
            content = walker.all(request, self)
        else: # latest n
            title = templates.blog_title_latest % locals()
            content = walker.recent(request, self, number)

        self.title = title
        self.html = content



class TopBlogPagePart(BlogPagePart):


    def __init__(self, resource_id, resource_manager, request, lfs, config, year=None, month=None, day=None, number=None):
        BlogPagePart.__init__(self, resource_id, resource_manager, request, lfs, config, year, month, day, number)
        
        page_part = resource_manager.get(resource_id, request, self)
        
        if page_part.exists():
            self.html = page_part.get_html() + self.html    

        
    def is_read_only(self):
        return False



class HTMLFormatter:
    
    
    def __init__(self, resource_manager, config):
    
        self.resource_manager = resource_manager
        
        self.site_url    = config.get("blog", "site_url")
        self.blog_path   = config.get("blog", "blog_path")


    def header(self):
        return ""
        
        
    def footer(self):
        return ""
        
        
    def no_entries(self):
        return templates.blog_no_entries % locals()
        

    def entry(self, year, month, day, entry, request, main_resource):
        page_key = "%s/%s/%s/%s/%s" % (self.blog_path, year, month, day, entry)
        page = self.resource_manager.get(page_key, request, main_resource)
        body = page.get_html()
        title = page.get_title()
        date = "%s/%s/%s" % (year, month, day)
        html_link = self.site_url + page_key

        return templates.blog_entry % locals()



class Walker:

    def __init__(self, formatter, blog_path, lfs):
        self.formatter   = formatter
        self.blog_path   = blog_path
        self.lfs         = lfs

    def all(self, request, main_resource):
        return \
            self.formatter.header() + \
            self.descend(self.blog_path, lambda year: self.year(year, request, main_resource)) + \
            self.formatter.footer()

    # contributed by Min Sik Kim
    # @@@ could be merged with descend with an extra parameter
    def recent(self, request, main_resource, num):
        import os

        content = ""
        years = self.lfs.get_children(self.blog_path)[0]
        for year in years:
            year_dir = os.path.join(self.blog_path, year)
            months = self.lfs.get_children(year_dir)[0]
            for month in months:
                month_dir = os.path.join(year_dir, month)
                days = self.lfs.get_children(month_dir)[0]
                for day in days:
                    day_dir = os.path.join(month_dir, day)
                    entries = self.lfs.get_children(day_dir)[1]
                    for entry in entries:
                        content += self.entry(year, month, day, entry, request, main_resource)
                        num -= 1
                        if num == 0: break
                    if num == 0: break
                if num == 0: break
            if num == 0: break
        return self.formatter.header() + content + self.formatter.footer()

    def year(self, year, request, main_resource):
        return self.descend(self.blog_path + "/" + year, lambda month: self.month(year, month, request, main_resource))
        
    def month(self, year, month, request, main_resource):
        return self.descend("%s/%s/%s" % (self.blog_path, year, month), lambda day: self.day(year, month, day, request, main_resource))
        
    def day(self, year, month, day, request, main_resource):
        return self.descend("%s/%s/%s/%s" % (self.blog_path, year, month, day), lambda entry: self.entry(year, month, day, entry, request, main_resource))
        
    def entry(self, year, month, day, entry, request, main_resource):
        return self.formatter.entry(year, month, day, entry, request, main_resource)
    
    def descend(self, key, func):
        directories, files = self.lfs.get_children(key)
        children = directories + files
        
        content = ""
        if children:
            for child in children:
                content += func(child)
        else:
            content = self.formatter.no_entries()
            
        return content
