#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


__provider__      = "CategoryProvider"
__contributions__ = {"edit_form": "category_edit_form", 
                     "draft_hidden": "category_draft_hidden",
                     "page_end": "category_page_end",
                     "post_processing": "category_post_processing"}    


from web_resource import PagePart

from providers.page.page_provider import HTMLPage, ForbiddenPage


def category_edit_form(request, resource, main_resource, config):

    if resource:
        categories = " ".join(resource.get_property("categories", []))
    else:
        categories = ""
        
    # @@@ templatize
    
    return """
    Categories <input type="text" name="categories" value="%(categories)s"/>
    <br/>
    """ % locals()


def category_draft_hidden(request, resource, main_resource, config):

    if resource:
        categories = " ".join(resource.get_property("categories", []))
    else:
        categories = ""
        
    # @@@ templatize
    
    return """
    <input type="hidden" name="categories" value="%(categories)s"/>
    """ % locals()


# @@@ templatize

def category_page_end(request, resource, main_resource, config):
    if main_resource == None:
        s = ""
        categories = ""
        prefix = config.get("DEFAULT", "home_href")
        for category in resource.get_property("categories", []):
            href = prefix + category
            categories += """ <a href="%(href)s">%(category)s</a>""" % locals()
        if categories:
            s +=  "<p><b>Categories</b>: %(categories)s</p>" % locals()
        return s
    else:
        return ""
    

def category_post_processing(request, resource, main_resource, config):
    categories = request.fields.getvalue("categories", "").split()
    if categories and request.logged_in():
        resource.set_property("categories", categories)

    return ""
    

    
class CategoryProvider:


    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config


    def get(self, resource_id, request, main_resource):
        if not resource_id.startswith("category"):
            return None

        page_part = CategoryPagePart(resource_id, self.lfs, self.config)
        #return HTMLPage(resource_id, page_part, self.resource_manager, request, main_resource, self.config)
        return page_part
        

    def post(self, resource_id, request, main_resource):
        return None
            


class CategoryPagePart(PagePart):


    def __init__(self, resource_id, lfs, config):

        PagePart.__init__(self, resource_id)

        parts = resource_id.split("/")

        prefix = config.get("DEFAULT", "home_href")

        self.html = "<ul>"
        if len(parts) != 2:
            self.title = "Categories"
            for value, resources in lfs.find_property("categories").items():
                self.html += "<li>%(value)s<ul>" % locals()
                for resource in resources:
                    href = prefix + resource
                    self.html += """<li><a href="%(href)s">%(resource)s</a></li>""" % locals()
                self.html += "</ul></li>"
        else:
            category = parts[1]
            self.title = "Category '%(category)s'" % locals()
            for resource in lfs.find_property("categories")[category]:
                href = prefix + resource
                self.html += """<li><a href="%(href)s">%(resource)s</a></li>""" % locals()
        self.html += "</ul>"
