#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


__provider__      = "CommentProvider"
__contributions__ = {"edit_form": "comments_allowed_edit_form", 
                     "draft_hidden": "comments_allowed_draft_hidden", 
                     "page_end" : "comment_page_end",
                     "post_processing": "comments_allowed_post_processing"}    


import urllib
import re

from web_resource import PagePart

from providers.page.page_provider import HTMLPage, ForbiddenPage

import templates

def comments_allowed_edit_form(request, resource, main_resource, config):

    # @@@ templatize

    if resource:    
        allow_comments = resource.get_property("allow_comments")
    else:
        allow_comments = ""
    
    s = """
    Allow Comments? 
    <select name="allow_comments">
    """
    
    if allow_comments == "YES":
        s+= """
        <option selected="selected">YES</option>
        <option>NO</option>
        """
    else:
        s+= """
        <option>YES</option>
        <option selected="selected">NO</option>
        """
        
    s += """
    </select>
    <br/>
    """
    
    return s


def comments_allowed_draft_hidden(request, resource, main_resource, config):

    # @@@ templatize

    if resource:    
        allow_comments = resource.get_property("allow_comments")
    else:
        allow_comments = ""
    
    s = """
    <input type="hidden" name="allow_comments" value="%(allow_comments)s" />
    """ % locals()
    
    return s


# @@@ templatize

def comment_page_end(request, resource, main_resource, config):
    if main_resource == None and resource.get_property("allow_comments", "NO") != "NO":
        enclosures = resource.enclosures("comment")
        html = """
        <h4>Comments (%s)</h4>
        """ % len(enclosures)
        for comment in enclosures:
            author_name = comment.get_property("author_name") or "unknown"
            author_link = comment.get_property("author_link")
            content = comment.get_content()

            # @@@ utility function
            author_name = re.sub(r'&', '&amp;', author_name)
            author_name = re.sub(r'<', '&lt;', author_name)
            author_name = re.sub(r'>', '&gt;', author_name)

            if author_link:
                author_link = re.sub(r'&', '&amp;', author_link)
                author_link = re.sub(r'<', '&lt;', author_link)
                author_link = re.sub(r'>', '&gt;', author_link)
                author_link = re.sub(r'"', '&quot;', author_link)

            content = re.sub(r'&', '&amp;', content)
            content = re.sub(r'<', '&lt;', content)
            content = re.sub(r'>', '&gt;', content)

            lastmod = comment.get_lastmod()
            if lastmod:
                import time
                lastmod_display = time.strftime("%A %d %B, %Y", time.gmtime(lastmod))
            else:
                lastmod_display = ""
            if author_link:
                citation = """<a rel="nofollow" href="%(author_link)s">%(author_name)s</a>""" % locals()
            else:
                citation = author_name
            html += """
            <div class="comment">
            <p class="comment_citation">%(citation)s on %(lastmod_display)s:</p>
            <div class="comment_content">%(content)s</div>
            </div>
            """ % locals()
        html += """
        <h4>Add a Comment</h4>
        <form class="add_comment" action="" method="POST">
          <table>
            <tr>
              <th>Name</th>
              <td><input type="text" name="author_name"/></td>
            </tr>
            <tr>
              <th>URI</th>
              <td><input type="text" name="author_link"/></td>
            </tr>
            <tr>
              <th>Comment</th>
              <td>
                <textarea name="comment_content" rows="10" cols="40"></textarea>
              </td>
            </tr>
            <tr>
              <td colspan="2">
                <input type="submit" value="Submit comment"/>
              </td>  
            </tr>      
          </table>    
        </form>
        """
    else:
        html = ""
    return html
    
    
def comments_allowed_post_processing(request, resource, main_resource, config):
    allow_comments = request.fields.getvalue("allow_comments")
    if allow_comments and request.logged_in():
        resource.set_property("allow_comments", allow_comments)

    return ""
    
    
    
class CommentProvider:


    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config


    def get(self, resource_id, request, main_resource):
        return None


    def post(self, resource_id, request, main_resource):

        comment_content = request.fields.getvalue("comment_content", None)

        if comment_content:
            return self.add_comment(resource_id, request, main_resource)
        else:
            return None

            
    def add_comment(self, resource_id, request, main_resource):

        comment_content = request.fields.getvalue("comment_content", None)
        author_name     = request.fields.getvalue("author_name", None)
        author_link     = request.fields.getvalue("author_link", None)

        inner_resource = self.resource_manager.get(resource_id, request, "hack")

        # @@@ could create comment object for this that wraps enclosure
        existing_comments = inner_resource.enclosures("comment")
        next_index = 0
        for comment in existing_comments:
            if int(comment.index) > next_index:
                next_index = int(comment.index)
        next_index += 1
        
        new_comment = inner_resource.enclosure("comment", next_index)
        new_comment.set_content(comment_content, "txt")
        if author_name:
            new_comment.set_property("author_name", author_name)
        if author_link:
            new_comment.set_property("author_link", author_link)

        # re-get to include new comment
        resource = self.resource_manager.get(resource_id, request, main_resource)

        return resource
