#
#    Copyright (C) 2003-2004  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
GETing the 'delete' resource confirms deletion of a resource.

POSTing to the 'delete' resource is intended as an alternative to DELETEing a
resource in cases like HTML forms where you can't directly DELETE.
"""


__provider__      = "DeleteProvider"
__contributions__ = {"menu": "delete_menu_part"}    


import urllib

from web_resource import PagePart

from providers.page.page_provider import HTMLPage, ForbiddenPage

import templates


def delete_menu_part(request, resource, main_resource, config):

    if request.logged_in() and not resource.is_read_only():
        delete_page_href = config.get("delete", "delete_href") + "?" + config.get("delete", "delete_resource_field") + "=" + urllib.quote(resource.get_id())
        menu_part = templates.delete_menu % locals()
    else:
        menu_part = ""
        
    return menu_part



class DeleteProvider:
    
    
    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config

        self.delete_path   = config.get("delete", "delete_path")
        self.resource_name = config.get("delete", "delete_resource_field")
        
       
    def get(self, resource_id, request, main_resource):

        if resource_id != self.delete_path:
            return None

        if not request.logged_in():
            return ForbiddenPage(resource_id, self.resource_manager, request, main_resource, self.config)

        resource_to_delete = request.fields.getvalue(self.resource_name)
        
        page_part = GetDeletePagePart(resource_id, resource_to_delete, self.resource_manager, request, self.config)

        return HTMLPage(resource_id, page_part, self.resource_manager, request, main_resource, self.config)


    def post(self, resource_id, request, main_resource):

        if resource_id != self.delete_path:
            return None

        if not request.logged_in():
            return ForbiddenPage(resource_id, self.resource_manager, request, main_resource, self.config)

        resource_to_delete = request.fields.getvalue(self.resource_name)
        
        page_part = PostDeletePagePart(resource_id, resource_to_delete, self.resource_manager, request, self.config)

        return HTMLPage(resource_id, page_part, self.resource_manager, request, main_resource, self.config)



class GetDeletePagePart(PagePart):


    def __init__(self, resource_id, resource_to_delete, resource_manager, request, config):
        PagePart.__init__(self, resource_id)
        
        site_title = config.get("page", "site_title")
        post_action = config.get("delete", "delete_href")
        resource_name = config.get("delete", "delete_resource_field")
        cancel_action = urllib.quote(resource_to_delete)

        if resource_to_delete:
            page = resource_manager.get(resource_to_delete, request, self)
            
            if not page.exists():
                self.title = templates.delete_page_not_exist_title % locals()
                self.html = templates.delete_page_not_exist_content % locals()
            elif page.is_read_only():
                self.title = templates.delete_read_only_title % locals()
                self.html = templates.delete_read_only_content % locals()
            else:
                self.title = templates.delete_confirm_title % locals()
                self.html = templates.delete_confirm_form % locals()
        else:
            self.title = templates.delete_missing_id_title % locals()
            self.html = templates.delete_missing_id_content % locals()



class PostDeletePagePart(PagePart):


    def __init__(self, resource_id, resource_to_delete, resource_manager, request, config):
        PagePart.__init__(self, resource_id)

        site_title = config.get("page", "site_title")

        if resource_to_delete:
            page = resource_manager.get(resource_to_delete, request, self)
            
            if not page.exists():
                self.title = templates.delete_page_not_exist_title % locals()
                self.html = templates.delete_page_not_exist_content % locals()
            elif page.is_read_only():
                self.title = templates.delete_read_only_title % locals()
                self.html = templates.delete_read_only_content % locals()
            else:
                page.delete()
                self.title = templates.delete_done_title % locals()
                self.html = templates.delete_done_content % locals()
        else:
            self.title = templates.delete_missing_id_title % locals()
            self.html = templates.delete_missing_id_form % locals()
