#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


__provider__      = "EditFormProvider"
__contributions__ = {"menu": "edit_menu_part"}    


import urllib

from web_resource import PagePart

from providers.page.page_provider import HTMLPage, ForbiddenPage

import templates

def edit_menu_part(request, resource, main_resource, config):

    if request.logged_in() and not resource.is_read_only():
        edit_page_href = config.get("editform", "edit_href") + "?" + config.get("editform", "edit_resource_field") + "=" + urllib.quote(resource.get_id())
        menu_part = templates.edit_menu % locals()
    else:
        menu_part = ""
        
    return menu_part



class EditFormProvider:
    
    
    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config

        self.edit_path          = config.get("editform", "edit_path")
        self.resource_name      = config.get("editform", "edit_resource_field")

       
    def get(self, resource_id, request, main_resource):

        if resource_id != self.edit_path:
            return None

        if not request.logged_in():
            return ForbiddenPage(resource_id, self.resource_manager, request, main_resource, self.config)

        resource_to_edit = request.fields.getvalue(self.resource_name)

        outer_page_part = EditFormPagePart(resource_id, resource_to_edit, self.resource_manager, request, self.config)
        
        html_page = HTMLPage(resource_id, outer_page_part, self.resource_manager, request, main_resource, self.config)
        html_page.expiry = 0
        
        return html_page


    def post(self, resource_id, request, main_resource):
        return None
        

            
class EditFormPagePart(PagePart):


    def __init__(self, resource_id, resource_to_edit, resource_manager, request, config):
    
        PagePart.__init__(self, resource_id)

        post_action = config.get("put", "put_href")
        page_content_name = config.get("put", "put_content_field")
        page_content_type_name = config.get("put", "put_content_type_field")
        resource_name = config.get("put", "put_resource_field")
        title_name = config.get("editform", "edit_title_field")
        cancel_action = urllib.quote(resource_to_edit)

        if resource_to_edit:
            page = resource_manager.get(resource_to_edit, request, self)
    
            if page.is_read_only():
                self.title = templates.edit_read_only_title % locals()
                self.html = templates.edit_read_only_title % locals()
            
            else:
        
                time = None
                site_title = config.get("page", "site_title")
                
        
                if page.exists():
                    page_title = page.get_title()
                    title = templates.page_title % locals()
                    page_content = page.get_content()
                    page_content_type = page.get_content_type()
                    time = page.get_lastmod()
                else: # new wiki word
                    page_title = resource_to_edit
                    title = templates.new_page_title % locals()
                    page_content = templates.new_page_content % locals()
                    page_content_type = config.get("editform", "editform_default_type")
                
                form_extras = ""
                for provider in resource_manager.get_contributors("edit_form"):
                    form_extras += provider(request, page, None, config)
                    
                content = templates.edit_form % locals()
        
        
                self.title = title
                self.html  = content
        else:
            self.title = templates.edit_missing_id_title % locals()
            self.html = templates.edit_missing_id_content % locals()            
