#
#    Copyright (C) 2003-2004  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


__provider__      = "EnclosureProvider"
__contributions__ = {}    


import re
import mimetypes

from formatters.manager import FormatterManager
from web_resource import TopLevelResource, FileObjectResource

from providers.page.page_provider import NonWikiPage, WikiPagePart, HTMLPage, HTMLPage404
    
# @@@ this is now too much like PageProvider not to be refactored
class EnclosureProvider:
    
    
    def __init__(self, resource_manager, lfs, config):
    
        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config
        
        self.formatter_manager = FormatterManager(config)
        
        
    def get(self, resource_id, request, main_resource):
    
        match = re.match("(.*)/__(\w+)__(\d+)", resource_id)
        if not match:
            return None
        
        base = match.group(1)
        enctype = match.group(2)
        index = match.group(3)
        
        key = resource_id
        
        enclosure = self.lfs.get(base).enclosure(enctype, index)
        content_type = enclosure.get_content_type()
            
        if enclosure.exists() and not self.formatter_manager.known(content_type):
            mime_type, encoding = mimetypes.guess_type("dummy.%s" % content_type)
            if not mime_type:
                mime_type = "application/octet-stream"
            return NonWikiPage(key, enclosure, mime_type)

        page_part = EnclosurePagePart(key, self.resource_manager, request, main_resource, self.config, enclosure, self.formatter_manager, base)
        
        if main_resource: # if there's a parent resource, we're done
            return page_part

        if page_part.exists():
            return HTMLPage(key, page_part, self.resource_manager, request, main_resource, self.config)
        else:
            return HTMLPage404(resource_id)


    # @@@ superclass should do this
    def post(self, resource_id, request, main_resource): 
        return None
        
        

class EnclosurePagePart(WikiPagePart):


    def __init__(self, resource_id, resource_manager, request, main_resource, config, file_object, formatter_manager, base_resource):
    
        WikiPagePart.__init__(self, resource_id, resource_manager, request, main_resource, config, file_object, formatter_manager)
        self.base_resource = base_resource


    def get_html(self):
        base_link = self.config.home_href + self.base_resource
        # @@@ templatize
        return """<p><a href="%s">Return to parent</a>.</p>""" % base_link + WikiPagePart.get_html(self)