#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


provider_list = [
    "providers.enclosure",
    "providers.login_logout",
    "providers.category",
    "providers.trackback",
    "providers.comment",
    "providers.editform",
    "providers.put",
    "providers.delete",
    "providers.blog",
    "providers.atom",
    "providers.draft",
    "providers.blogcalendar",
    "providers.upload",
    "providers.menu",
    "providers.css",
    "providers.propertylist",
    "providers.page",
]


import filesystem



class ProviderManager:


    def __init__(self, config):
    
        # initialise config
        for package_name in provider_list:
            config.add(package_name)
        config.read_main()
        
        self.providers = []
        self.contributors = {}

        lfs_root = config.get("DEFAULT", "lfs_root")
        lfs = filesystem.LeonardoFileSystem(lfs_root)

        # register resource providers

        for provider_package_name in provider_list:

            provider_module_name = provider_package_name + "." + provider_package_name.split('.')[-1] + "_provider"

            provider_module = get_module(provider_module_name)
            if provider_module.__provider__:
                self.providers.append(getattr(provider_module, provider_module.__provider__)(self, lfs, config))
            for key, contribution_function in provider_module.__contributions__.items():
                self.contributors.setdefault(key, []).append(getattr(provider_module, contribution_function))
        
    
    def get_contributors(self, key):
    
        return self.contributors.get(key, [])
        
        
    def get(self, resource_id, request, main_resource):
        """
        Attempts to retrieve the resource with the given resource_id.
        
        request is also passed in to provide authentication information and
        HTTP parameters. @@@ This could be improved.
        
        main_resource is None if the requested resource is what the client
        directly requested. Otherwise it is the actual resource object
        requested by the client (and the resource in resource_id is just
        a component being retrieved to contribute to the the main_resource).
        """
        
        for provider in self.providers:
            resource = provider.get(resource_id, request, main_resource)
            if resource:
                return resource
        return None
        
        
    def post(self, resource_id, request, main_resource):
    
        for provider in self.providers:
            resource = provider.post(resource_id, request, main_resource)
            if resource:
                return resource
        return None
        
        
def get_module(name):
    module = __import__(name)
    components = name.split('.')
    for comp in components[1:]:
        module = getattr(module, comp)
    return module
