#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
This just a little sample provider that lists all the properties across the
site, their values and the resources that have them.
"""


__provider__      = "PropertyListProvider"
__contributions__ = {}    


from web_resource import PagePart

from providers.page.page_provider import HTMLPage, ForbiddenPage



class PropertyListProvider:


    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config


    def get(self, resource_id, request, main_resource):

        if not resource_id.startswith("propertylist"):
            return None

        page_part = PropertyListPagePart(resource_id, self.lfs)
        return HTMLPage(resource_id, page_part, self.resource_manager, request, main_resource, self.config)


class PropertyListPagePart(PagePart):


    def __init__(self, resource_id, lfs):

        PagePart.__init__(self, resource_id)

        self.title = "Property List"
        
        self.html = "<ul>"
        for prop in lfs.list_properties():
            self.html += "<li>%(prop)s<ul>" % locals()
            for value, resources in lfs.find_property(prop).items():
                self.html += "<li>%(value)s<ul>" % locals()
                for resource in resources:
                    self.html += "<li>%(resource)s</li>" % locals()
                self.html += "</ul></li>"
            self.html += "</ul></li>"
        self.html += "</ul>"