#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


__provider__      = "TrackbackProvider"
__contributions__ = {"edit_form": "trackbacks_allowed_edit_form",
                     "draft_hidden": "trackbacks_allowed_draft_hidden",
                     "page_end" : "trackback_page_end",
                     "post_processing": "trackbacks_allowed_post_processing"}    

import urllib

from web_resource import TopLevelResource, PagePart

from providers.page.page_provider import HTMLPage, ForbiddenPage

import templates


def trackbacks_allowed_edit_form(request, resource, main_resource, config):

    # @@@ templatize

    if resource:    
        allow_trackbacks = resource.get_property("allow_trackbacks")
    else:
        allow_trackbacks = ""
    
    s = """
    Allow Trackbacks? 
    <select name="allow_trackbacks">
    """
    
    if allow_trackbacks == "YES":
        s+= """
        <option selected="selected">YES</option>
        <option>NO</option>
        """
    else:
        s+= """
        <option>YES</option>
        <option selected="selected">NO</option>
        """
        
    s += """
    </select>
    <br/>
    """
    
    return s


def trackbacks_allowed_draft_hidden(request, resource, main_resource, config):

    # @@@ templatize

    if resource:    
        allow_trackbacks = resource.get_property("allow_trackbacks")
    else:
        allow_trackbacks = ""
    
    s = """
    <input type="hidden" name="allow_trackbacks" value="%(allow_trackbacks)s" />
    """ % locals()
    
    return s


# @@@ templatize

def trackback_page_end(request, resource, main_resource, config):
    if main_resource == None and resource.get_property("allow_trackbacks", "NO") != "NO":
        enclosures = resource.enclosures("trackback")
        html = """
        <h4>Trackbacks (%s)</h4>
        """ % len(enclosures)
        for trackback in enclosures:
            url = trackback.get_property("url")
            title = trackback.get_property("title") or url
            blog_name = trackback.get_property("blog_name") or "unknown"
            content = trackback.get_content()

            # @@@ escape characters from content
            lastmod = trackback.get_lastmod()
            if lastmod:
                import time
                lastmod_display = time.strftime("%A %d %B, %Y", time.gmtime(lastmod))
            else:
                lastmod_display = ""
            citation = """<a rel="nofollow" href="%(url)s">%(title)s</a>""" % locals()
            html += """
            <div class="trackback">
            <p class="trackback_citation">%(citation)s from %(blog_name)s on %(lastmod_display)s:</p>
            <div class="trackback_content">%(content)s</div>
            </div>
            """ % locals()
    else:
        html = ""
    return html
    
    
def trackbacks_allowed_post_processing(request, resource, main_resource, config):
    allow_trackbacks = request.fields.getvalue("allow_trackbacks")
    if allow_trackbacks and request.logged_in():
        resource.set_property("allow_trackbacks", allow_trackbacks)

    return ""


    
class TrackbackProvider:


    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config


    def get(self, resource_id, request, main_resource):
        return None
        

    def post(self, resource_id, request, main_resource):

        url = request.fields.getvalue("url", None)

        if url:
            return self.add_trackback(resource_id, request, main_resource)
        else:
            return None


    def add_trackback(self, resource_id, request, main_resource):

        title     = request.fields.getvalue("title", None)
        excerpt   = request.fields.getvalue("excerpt", None)
        url       = request.fields.getvalue("url", None)
        blog_name = request.fields.getvalue("blog_name", None)

        inner_resource = self.resource_manager.get(resource_id, request, "hack")

        # @@@ should this be wrapped by a trackback object
        existing_trackbacks = inner_resource.enclosures("trackback")
        next_index = 0
        for trackback in existing_trackbacks:
            if int(trackback.index) > next_index:
                next_index = int(trackbaack.index)
        next_index += 1
        
        new_trackback = inner_resource.enclosure("trackback", next_index)
        new_trackback.set_content(excerpt, "txt")
        if title:
            new_trackback.set_property("title", title)
        if url:
            new_trackback.set_property("url", url)
        if blog_name:
            new_trackback.set_property("blog_name", blog_name)

        return TrackbackResultResource(resource_id)
        
        

class TrackbackResultResource(TopLevelResource):


    def __init__(self, resource_id):
        TopLevelResource.__init__(self, resource_id, "text/xml")
        
        
    def get_content(self):
        return """
        <response>
          <error>0</error>
        </response>
        """