#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


__provider__      = "UploadFormProvider"
__contributions__ = {"menu": "upload_menu_part"}    


import urllib

from web_resource import PagePart

from providers.page.page_provider import HTMLPage, ForbiddenPage

import templates


def upload_menu_part(request, resource, main_resource, config):

    if request.logged_in():
        upload_href = config.get("upload", "upload_href")
        menu_part = templates.upload_menu % locals()
    else:
        menu_part = ""
        
    return menu_part



class UploadFormProvider:
    
    
    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config

        self.upload_path   = config.get("upload", "upload_path")

       
    def get(self, resource_id, request, main_resource):

        if resource_id != self.upload_path:
            return None

        if not request.logged_in():
            return ForbiddenPage(resource_id, self.resource_manager, request, self.config)

        page_part = UploadPagePart(resource_id, self.resource_manager, request, self.config)
        
        html_page = HTMLPage(resource_id, page_part, self.resource_manager, request, main_resource, self.config)
        
        return html_page
        
        
    def post(self, resource_id, request, main_resource):
    
        return None
        

            
class UploadPagePart(PagePart):


    def __init__(self, resource_id, resource_manager, request, config):

        PagePart.__init__(self, resource_id)

        action = config.get("put", "put_href")
        file_content_name = config.get("put", "put_content_field")
        file_content_type_name = config.get("put", "put_content_type_field")
        path_name = config.get("put", "put_resource_field")

        site_title = config.get("page", "site_title")
        self.title = templates.upload_title % locals()
        
        content = templates.upload_form % locals()
        self.html  = content
